/*************************************************************************************
 *
 *	Simple Speech: a Speech Synthesizer and Voice Recognizer for Processing
 *
 *	v2.0 - 10/23/2018
 *
 *	(c) 2018 -  Rolf van Gelder, CAGE Web Design, Eindhoven, NL (cagewebdev.com)
 *
 *************************************************************************************/
package simpleSpeech;

import processing.core.*;

import com.sun.speech.freetts.Voice;
import com.sun.speech.freetts.VoiceManager;
import com.sun.speech.freetts.FreeTTSSpeakable;
import com.sun.speech.freetts.FreeTTSSpeakableImpl;

import java.util.Locale;

import javax.speech.Central;
import javax.speech.Engine;
import javax.speech.EngineList;
import javax.speech.synthesis.Synthesizer;
import javax.speech.synthesis.SynthesizerModeDesc;
import javax.speech.synthesis.SynthesizerProperties;

/*************************************************************************************
 *
 *	Speak: a Class for speech synthesis
 *
 *************************************************************************************/
public class Speak {
	PApplet          parent;
	VoiceManager     myVM;
	Voice            myVoice;
	Synthesizer      speechSynthesizer;
	FreeTTSSpeakable mySpeakable;
	
	// VERSION OF THE LIBRARY
	String version     = "2.0";
	String releaseDate = "10/23/2018";	

	/**********************************************************************************
	 *
	 *	Constructor: instantiate the class in procesing's standard way: pass "this" as
	 *	a parameter,<br />
	 *	when instantiating in a Processing applet<br />
	 *	It uses the default voice (<b>kevin16</b>)
	 *
	 *	@param parent	Processing applet
	 *
	 **********************************************************************************/	
	public Speak(PApplet parent) {
		initSpeak(parent,null);
	} // Speak()
	

	/**********************************************************************************
	 *
	 *	Constructor (overload): instantiate the class in procesing's standard way:
	 *	pass "this" as a parameter,<br />
	 *	when instantiating in a Processing applet<br />
	 *	<b>voiceName</b> selects the voice to use (<b>kevin</b> or <b>kevin16</b>)
	 *
	 *	@param parent		Processing applet
	 *	@param voiceName	Voice to use
	 *
	 **********************************************************************************/		 
	public Speak(PApplet parent, String voiceName) {
		initSpeak(parent,voiceName);
	} // Speak()
	

	/**********************************************************************************
	 *
	 *	Initialize the synthesizer
	 *
	 *	@param parent		Processing applet
	 *	@param voiceName	Voice to use
	 *
	 **********************************************************************************/
	private void initSpeak(PApplet parent, String voiceName) {
		this.parent = parent;
		parent.registerMethod("dispose", this);
		
		myVM = VoiceManager.getInstance();
		
		if (myVM == null) {
			System.err.println("Voice manager not available.");
			System.exit(1);
		} // if (myVM == null)
		
		if (voiceName == null) voiceName = "kevin16";
		
		myVoice = myVM.getVoice(voiceName);
		
		if (myVoice==null) {
			System.err.println("Default voice not available.");
			System.exit(1);
		} // if (myVoice==null)
			
		myVoice.allocate();
	} // initSpeak()

	
	/**********************************************************************************
	 *
	 *	Speaking text from a string
	 *
	 *	@param _s		Text to Speak
	 *
	 **********************************************************************************/	 
	public void speak(String _s) {
		mySpeakable = new FreeTTSSpeakableImpl(_s);
		myVoice.speak(mySpeakable);
	} // speak()


	/**********************************************************************************
	 *
	 *	Speak and Wait
	 *
	 *	@param _s		Text to Speak
	 *
	 **********************************************************************************/		
	public void speakAndWait(String _s) {
		mySpeakable = new FreeTTSSpeakableImpl(_s);
		myVoice.speak(mySpeakable);
		while(isSpeaking());
		//System.err.println(mySpeakable.isCompleted());
	} // speakAndWait()
	

	/**********************************************************************************
	 *
	 *	Is the Voice still Speaking?
	 *
	 *	@param _s		Text to Speak
	 *
	 **********************************************************************************/		
	public boolean isSpeaking() {
		boolean b = false;
		try {
			b = !mySpeakable.isCompleted();
		} catch (Exception e) {
		}
		return b;
	} // isSpeaking()

	
	/*********************************************************************************
	 *
	 *	List the available Voices in the Console
	 *
	 *********************************************************************************/
	public void listVoices() {
		Voice[] voices = myVM.getVoices();
		System.out.println("\nSimpleSpeech - Available voices:");
		for (int i = 0; i < voices.length; i++) {
			System.out.println(voices[i].getName() + " (" + voices[i].getDomain() + " domain)");
		} // for (int i = 0; i < voices.length; i++)
	} // listVoices()
	
	
	/*********************************************************************************
	 *
	 *	Displays Version Information of this Library in the Console
	 *
	 *********************************************************************************/
	public void version() {
		System.out.println("\nSimpleSpeech v" + version + " [" + releaseDate + "], by Rolf van Gelder (cagewebdev.com)\n");
	} // void version()	


	/**********************************************************************************
	 *
	 *	Dispose the Voice
	 *
	 *	@param _s		Text to Speak
	 *
	 **********************************************************************************/	    
	public void dispose() {
		try {
			myVoice.deallocate();
		} catch (Exception e) {
		}
	} // dispose()
} // Speak
